package io.flutter.plugins.urllauncher;

import android.app.Activity;
import android.app.Application;
import android.content.ActivityNotFoundException;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.provider.Browser;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

/**
 * Launches components for URLs.
 */
class UrlLauncher {
    private final Context applicationContext;
    @Nullable
    private Activity mCurrentActiveActivity;

    /**
     * Uses the given {@code applicationContext} for launching intents.
     *
     * <p>It may be null initially, but should be set before calling {@link #launch}.
     */
    UrlLauncher(Context applicationContext, @Nullable Activity activity) {
        this.applicationContext = applicationContext;
        this.mCurrentActiveActivity = activity;
        if (activity != null) {
            abActivity(activity);
        }
    }

    void setmCurrentActiveActivity(@Nullable Activity activity) {
        if (this.mCurrentActiveActivity != null || activity == null) {
            return;
        }
        this.mCurrentActiveActivity = activity;
        abActivity(activity);
    }

    void abActivity(Activity activity) {
        activity.getApplication().registerActivityLifecycleCallbacks(new Application.ActivityLifecycleCallbacks() {
            @Override
            public void onActivityCreated(Activity activity, Bundle savedInstanceState) {
                mCurrentActiveActivity = activity;
            }

            @Override
            public void onActivityStarted(Activity activity) {

                mCurrentActiveActivity = activity;
            }

            @Override
            public void onActivityResumed(Activity activity) {

                mCurrentActiveActivity = activity;
            }

            @Override
            public void onActivityPaused(Activity activity) {
            }

            @Override
            public void onActivityStopped(Activity activity) {

                if (mCurrentActiveActivity == activity) {
                    mCurrentActiveActivity = null;
                }
            }

            @Override
            public void onActivitySaveInstanceState(Activity activity, Bundle outState) {

            }

            @Override
            public void onActivityDestroyed(Activity activity) {
                if (mCurrentActiveActivity == activity) {
                    mCurrentActiveActivity = null;
                }
            }
        });
    }

    /**
     * Returns whether the given {@code url} resolves into an existing component.
     */
    boolean canLaunch(String url) {
        Intent launchIntent = new Intent(Intent.ACTION_VIEW);
        launchIntent.setData(Uri.parse(url));
        ComponentName componentName =
                launchIntent.resolveActivity(applicationContext.getPackageManager());

        return componentName != null
                && !"{com.android.fallback/com.android.fallback.Fallback}"
                .equals(componentName.toShortString());
    }

    /**
     * Attempts to launch the given {@code url}.
     *
     * @param headersBundle    forwarded to the intent as {@code Browser.EXTRA_HEADERS}.
     * @param useWebView       when true, the URL is launched inside of {@link WebViewActivity}.
     * @param enableJavaScript Only used if {@param useWebView} is true. Enables JS in the WebView.
     * @param enableDomStorage Only used if {@param useWebView} is true. Enables DOM storage in the
     * @return {@link LaunchStatus#NO_ACTIVITY} if there's no available {@code applicationContext}.
     * {@link LaunchStatus#ACTIVITY_NOT_FOUND} if there's no activity found to handle {@code
     * launchIntent}. {@link LaunchStatus#OK} otherwise.
     */
    LaunchStatus launch(
            String url,
            Bundle headersBundle,
            boolean useWebView,
            boolean enableJavaScript,
            boolean enableDomStorage) {
        if (mCurrentActiveActivity == null) {
            return LaunchStatus.NO_ACTIVITY;
        }

        Intent launchIntent;
        if (useWebView) {
            launchIntent =
                    WebViewActivity.createIntent(
                            mCurrentActiveActivity, url, enableJavaScript, enableDomStorage, headersBundle);
        } else {
            launchIntent =
                    new Intent(Intent.ACTION_VIEW)
                            .setData(Uri.parse(url))
                            .putExtra(Browser.EXTRA_HEADERS, headersBundle);
        }

        try {
            mCurrentActiveActivity.startActivity(launchIntent);
        } catch (ActivityNotFoundException e) {
            return LaunchStatus.ACTIVITY_NOT_FOUND;

        }


        return LaunchStatus.OK;
    }

    /**
     * Closes any activities started with {@link #launch} {@code useWebView=true}.
     */
    void closeWebView() {
        applicationContext.sendBroadcast(new Intent(WebViewActivity.ACTION_CLOSE));
    }

    /**
     * Result of a {@link #launch} call.
     */
    enum LaunchStatus {
        /**
         * The intent was well formed.
         */
        OK,
        /**
         * No activity was found to launch.
         */
        NO_ACTIVITY,
        /**
         * No Activity found that can handle given intent.
         */
        ACTIVITY_NOT_FOUND,
    }
}
